/***********************************************************************************************************************
 *
 * blueBill Resources - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it
 * SCM: https://java.net/hg/bluebill~resources-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.factsheet.xenocanto;

import javax.annotation.Nonnull;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FilterInputStream;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.Writer;
import it.tidalwave.util.logging.Logger;
import org.w3c.tidy.Configuration;
import org.w3c.tidy.Tidy;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class TidyFilterInputStream extends FilterInputStream
  {
    private static final String CLASS = TidyFilterInputStream.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private static final PrintWriter LOG_PRINT_WRITER = new PrintWriter(new Writer()
      {
        private StringBuilder buffer = new StringBuilder();
        
        @Override
        public void write (char[] chars, int from, int lenght)
          {
            for (int i = from; i < from + lenght; i++)
              {
                if (chars[i] == 13)
                  {
                    logger.fine(buffer.toString());
                    buffer = new StringBuilder();
                  }
                else
                  {
                    buffer.append(chars[i]);
                  }
              }
          }

        @Override
        public void flush()
          {
            logger.fine(buffer.toString());
          }

        @Override
        public void close()
          {
            flush();
          }
      });

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public TidyFilterInputStream (final @Nonnull InputStream in)
      {
        super(createTidyStream(in));
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    private static InputStream createTidyStream (final @Nonnull InputStream is)
      {
        final ByteArrayOutputStream baos = new ByteArrayOutputStream();
        final Tidy tidy = new Tidy();
        tidy.setXHTML(true);
        tidy.setCharEncoding(Configuration.UTF8);
        tidy.setErrout(LOG_PRINT_WRITER);
        tidy.parse(is, baos);

        return new ByteArrayInputStream(baos.toByteArray());
      }
  }
