/***********************************************************************************************************************
 *
 * blueBill Resources - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it
 * SCM: https://java.net/hg/bluebill~resources-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.factsheet.xenocanto;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import java.util.HashMap;
import java.util.Map;
import java.io.ByteArrayInputStream;
import java.io.FilterInputStream;
import java.io.IOException;
import java.io.InputStream;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class XenoCantoFilterInputStream extends FilterInputStream
  {
    private static final Map<Integer, String> REPLACE_MAP = new HashMap<Integer, String>();

    private boolean doubleQuoteSection = false;

    @CheckForNull
    private InputStream aux;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    static
      {
        REPLACE_MAP.put((int)'<', "&lt;");
        REPLACE_MAP.put((int)'>', "&gt;");
        REPLACE_MAP.put((int)'&', "&amp;");
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public XenoCantoFilterInputStream (final @Nonnull InputStream delegate)
      {
        super(delegate);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public int read() throws IOException
      {
        int c = -1;

        if (aux != null)
          {
            c = aux.read();

            if (c < 0)
              {
                aux = null;
              }
            else
              {
                return c;
              }
          }

        c = super.read();

        if (c == '"')
          {
            doubleQuoteSection = !doubleQuoteSection;
          }
        else if (doubleQuoteSection)
          {
            final String replacement = REPLACE_MAP.get(c);

            if (replacement != null)
              {
                aux = new ByteArrayInputStream(replacement.getBytes());
              }

            if (aux != null)
              {
                c = aux.read();
              }
          }

        return c;
      }
  }
